#! /usr/bin/perl -w

#
# This script updates grub configuration and initrd images after the
# (un)installation of MicroSoft integration components. It is invoked
# in the post-(un)install scriptlet of the mshvic RPM.
#
# Q:
# - creation of /var/run/mshvic in this script? or in the RPM?
# - is there a better (more standard) way of getting the installed kernels?

use strict;
use diagnostics;

sub Usage(){
    print STDERR "Usage: $0 [ install | uninstall ]\n";
    exit 1;
}

Usage unless scalar(@ARGV) == 1;

my $action = $ARGV[0];
Usage unless $action eq "install" or $action eq "uninstall";

my $debug = 0;

# Check the directory that stores the state files

my $statedir = "/var/run/mshvic";
if (not -d $statedir){
    print STDERR "No directory $statedir found, exiting\n";
    exit 1;
}

# Get installed kernels

opendir(BOOT,"/boot") || die "can't opendir /boot: $!";
my @kernels = sort map {s/^vmlinuz-//;$_} grep {/^vmlinuz-(\S+)/} readdir(BOOT);
closedir BOOT;
print "\@kernels = @kernels\n" if $debug;

# Handle them

for my $kernel (@kernels){
    print "Processing kernel $kernel\n" if $debug;
    
    # Check state file
    if ($action eq "install"){
        next if -f "$statedir/$kernel";
    }else{
        next unless -f "$statedir/$kernel";
    }
    
    # Recreate initrd
    my $initrd = "/sbin/mkinitrd -f /boot/initrd-$kernel.img $kernel";
    print "  Now run \"$initrd\"\n" if $debug;
    if (system($initrd)){
        print "  Could not run $initrd: $!\n";
    }
    
    # Update grub configuration
    my $grubby = "/sbin/grubby --update-kernel=/boot/vmlinuz-$kernel ";
    $grubby .= ($action eq "install" ? "--args=" : "--remove-args=") . "\"hda=noprobe hdb=noprobe\"";
    print "  Now run \"$grubby\"\n" if $debug;
    if (system($grubby)){
        print "  Could not run $grubby: $!\n";
    }
    
    # Create a state file
    if ($action eq "install"){
        print "  Create file \"$statedir/$kernel\"\n" if $debug;
        system("/bin/touch $statedir/$kernel") && print "Could not create file \"$statedir/$kernel\"\n";
    }else{
        print "  Remove file \"$statedir/$kernel\"\n" if $debug;
        unlink "$statedir/$kernel";
    }
}

# Housekeeping: remove state file for kernels not found on the system

if (opendir(STATE,$statedir)){
    for my $file (grep { ! /^\./ && -f "$statedir/$_" } readdir(STATE)){
        unlink "$statedir/$file" unless grep {$file eq $_} @kernels;
    }
    closedir STATE;
}

# Done

exit 0;
